/**
 * @file
 * Drupal behavior for the edit_ui blocks.
 */

(function (Drupal, $) {
  "use strict";

  /**
   * Drupal edit_ui block behavior.
   */
  Drupal.behaviors.editUiBlock = {
    attach: function (context, settings) {
      if (!this.isInitialized) {
        this.isInitialized = true;
        var collection;
        var model;

        // Init block collection.
        collection = Drupal.editUiBlock.collections.blockCollection = new Drupal.editUiBlock.BlockCollection();

        // Init block views.
        Drupal.editUiBlock.views.bodyVisualView = new Drupal.editUiBlock.BodyVisualView({
          collection: collection
        });

        // Init block view list.
        Drupal.editUiBlock.views.blockVisualViews = [];
        Drupal.editUiBlock.views.ghostBlockVisualViews = [];

        // Global variable for new block.
        Drupal.editUiBlock.models.newBlockModel = null;

        // Init dragging block model and view.
        model = Drupal.editUiBlock.models.draggingBlockModel = new Drupal.editUiBlock.DraggingBlockModel();
        Drupal.editUiBlock.views.draggingBlockVisualView = new Drupal.editUiBlock.DraggingBlockVisualView({
          model: model,
          el: $('#edit-ui__dragging')
        });
        Drupal.editUiBlock.views.ghostBlockVisualView = new Drupal.editUiBlock.GhostBlockVisualView({
          model: model,
          el: $(Drupal.theme('editUiGhostBlock'))
        });
      }
    }
  };

  /**
   * edit_ui block Backbone objects.
   */
  Drupal.editUiBlock = {
    // A hash of View instances.
    views: {},
    // A hash of Collection instances.
    collections: {},
    // A hash of Model instances.
    models: {}
  };

  /**
   * Add new AJAX command called after the block as been created in the backend.
   */
  Drupal.AjaxCommands.prototype.editUiAddNewBlock = function (ajax, response, status) {
    if (Drupal.editUiBlock.models.newBlockModel instanceof Drupal.editUiBlock.BlockModel) {
      Drupal.editUiBlock.models.newBlockModel.set({
        id: response['id'],
        plugin_id: response['plugin_id'],
        label: response['label'],
        html_id: response['html_id'],
        provider: response['provider']
      });
      Drupal.editUiBlock.views.bodyVisualView.initBlock(
        Drupal.editUiBlock.models.newBlockModel,
        $(response.content)
      );
      Drupal.editUiBlock.models.newBlockModel = null;
    }
  };

  /**
   * Wrapper block template.
   *
   * @param Object settings
   *   An object containing the block attributes.
   * @param string content
   *   The content to be wrapped.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiWrapperBlock = function (settings, content) {
    return '' +
      '<div id="' + settings.html_id + '" class="edit-ui__block edit-ui__placeholder-block js-edit-ui__' + settings.plugin_id + '">' +
        content +
      '</div>';
  };

  /**
   * Placeholder block template.
   *
   * @param Object settings
   *   An object containing the block attributes.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiPlaceholderBlock = function (settings) {
    return '' +
      '<div id="block-' + settings.id + '" class="block block-' + settings.provider + '">' +
        '<h2>' + settings.label + '</h2>' +
      '</div>';
  };

  /**
   * Ghost block template.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiGhostBlock = function () {
    return '<div class="edit-ui__ghost"></div>';
  };

  /**
   * disabled block template.
   *
   * @param Object settings
   *   An object containing the block attributes.
   *
   * @return String
   *   A string representing a DOM fragment.
   */
  Drupal.theme.editUiDisabledBlock = function (settings) {
    return '<a href="#" class="edit-ui__toolbar__link">' + settings.label + '</a>';
  };

})(Drupal, jQuery);
