/**
 * @file
 * A Backbone Collection for the edit_ui region.
 */

(function (Drupal, Backbone, $) {

  "use strict";

  /**
   * Backbone collection for the edit_ui region.
   */
  Drupal.editUiRegion.RegionCollection = Backbone.Collection.extend({
    /**
     * {@inheritdoc}
     */
    model: Drupal.editUiRegion.RegionModel,

    /**
     * Initiliaze regions data.
     */
    init: function () {
      this.models.forEach(function (region) {
        region.trigger('init');
      });
    },

    /**
     * Find hovered region.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The dragged block.
     * @param Number x
     *   Mouse left position.
     * @param Number y
     *   Mouse top position.
     */
    drag: function (block, x, y) {
      var region = this.getActiveRegion();

      if (region) {
        // Trigger dragIn event only to active region.
        region.trigger('dragIn', block, {x: x, y: y});
      }
      else {
        // Trigger dragOut event to all regions.
        this.models.forEach(function (region) {
          region.trigger('dragOut', block, {x: x, y: y});
        });
      }
    },

    /**
     * Drop block.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The dropped block.
     * @param Number x
     *   Mouse left position.
     * @param Number y
     *   Mouse top position.
     */
    drop: function (block, x, y) {
      var region = this.getActiveRegion();

      // Drop block in region.
      if (region) {
        region.trigger('drop', block, {x: x, y: y});
      }
    },

    /**
     * Get region from given input.
     *
     * @param String region
     *   Region name.
     * @return Drupal.editUiRegion.RegionModel
     */
    getRegion: function (region) {
      return this.findWhere({region: region});
    },

    /**
     * Get the active region.
     *
     * @return Drupal.editUiRegion.RegionModel
     */
    getActiveRegion: function () {
      return this.findWhere({isActive: true});
    }
  });

}(Drupal, Backbone, jQuery));
